@@ -738,10 +738,18 @@ populate_contact_members (EContact *contact, gpointer data)
 		EVCardAttribute *attr;
 		EGroupMember *member;
 		member = (EGroupMember *) member_list->data;
+
 		attr = e_vcard_attribute_new (NULL, EVC_EMAIL);
+		e_vcard_attribute_add_param_with_value (attr,
+                                                        e_vcard_attribute_param_new (EVC_X_DEST_CONTACT_UID),
+							member->id);
 		e_vcard_attribute_add_param_with_value (attr,
                                                         e_vcard_attribute_param_new (EVC_X_DEST_EMAIL),
 							member->email);
+		if (member->name)
+			e_vcard_attribute_add_param_with_value (attr,
+                                                        e_vcard_attribute_param_new (EVC_X_DEST_NAME),
+							member->name);
 		e_vcard_attribute_add_value (attr, member->email);
 		e_vcard_add_attribute (E_VCARD (contact), attr);
 	}
@@ -753,7 +761,7 @@ static void
 set_members_in_gw_item (EGwItem  *item, EContact *contact, EBookBackendGroupwise *egwb)
 
 {
-  	GList*  members, *temp, *items;
+  	GList  *members, *temp, *items, *p, *emails_without_ids;
 	GList *group_members;
 	char *email;
 	EGwFilter *filter;
@@ -761,32 +769,51 @@ set_members_in_gw_item (EGwItem  *item, EContact *contact, EBookBackendGroupwise
 	char *id;
 	EGwItem *temp_item;
 	int count = 0;
+	EGroupMember *member;
 
-	members = e_contact_get (contact, E_CONTACT_EMAIL);
+	members = e_contact_get_attributes (contact, E_CONTACT_EMAIL);
 	temp = members;
 	filter = e_gw_filter_new ();
+	group_members = NULL;
+	emails_without_ids = NULL;
 	for ( ;temp != NULL; temp = g_list_next (temp)) {
-		email =  temp->data;
-		if (email) {
+		EVCardAttribute *attr = temp->data;
+		id = email = NULL;
+		for (p = e_vcard_attribute_get_params (attr); p; p = p->next) {
+			EVCardAttributeParam *param = p->data;
+			const char *param_name = e_vcard_attribute_param_get_name (param);
+			if (!g_ascii_strcasecmp (param_name,
+						 EVC_X_DEST_CONTACT_UID)) {
+				GList *v = e_vcard_attribute_param_get_values (param);
+				id = v ? v->data : NULL;
+			} else if (!g_ascii_strcasecmp (param_name,
+							EVC_X_DEST_EMAIL)) {
+				GList *v = e_vcard_attribute_param_get_values (param);
+				email = v ? v->data : NULL;
+			}
+		}
+		if (id) {
+			member = g_new0 (EGroupMember , 1);
+			member->id = g_strdup (id);
+			group_members = g_list_append (group_members, member);
+		} else if (email) {
 			e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_EQUAL, "emailList/@primary", email);
+			emails_without_ids = g_list_append (emails_without_ids, g_strdup (email));
 			count++;
 		}
 	       
 	}
 	e_gw_filter_group_conditions (filter, E_GW_FILTER_OP_OR, count);
 	items = NULL;
-	group_members = NULL;
-	temp = g_list_copy (members);
-	status = e_gw_connection_get_items (egwb->priv->cnc, egwb->priv->container_id, NULL, filter, &items);
+	if (count)
+		status = e_gw_connection_get_items (egwb->priv->cnc, egwb->priv->container_id, NULL, filter, &items);
 	for (; items != NULL; items = g_list_next (items )) {
-		EGroupMember *member;
 		GList *emails;
 		GList *ptr;
 		temp_item = E_GW_ITEM (items->data);
 		emails = e_gw_item_get_email_list (temp_item);
-		if (temp && (ptr = g_list_find_custom (temp, emails->data, (GCompareFunc)strcasecmp ))) {
-			
-			temp = g_list_remove_link (temp, ptr);
+		if (emails_without_ids && (ptr = g_list_find_custom (emails_without_ids, emails->data, (GCompareFunc)strcasecmp ))) {
+			emails_without_ids = g_list_remove_link (emails_without_ids, ptr);
 			g_list_free (ptr);
 			id = g_strdup (e_gw_item_get_id (temp_item));
 			member = g_new0 (EGroupMember , 1);
@@ -795,12 +822,11 @@ set_members_in_gw_item (EGwItem  *item, EContact *contact, EBookBackendGroupwise
 		}
 		g_object_unref (temp_item);
 	}
-	if (temp)
-		g_list_free (temp);
-	temp = members;
-	for ( ;temp != NULL; temp = g_list_next (temp)) 
-		g_free (temp->data);
+       
+	g_list_foreach (members, (GFunc) e_vcard_attribute_free, NULL);
 	g_list_free (members);
+	g_list_foreach (emails_without_ids, (GFunc) g_free, NULL);
+	g_list_free (emails_without_ids);
 	g_list_free (items);
        	e_gw_item_set_member_list (item, group_members);
   
@@ -1016,8 +1042,13 @@ fill_contact_from_gw_item (EContact *contact, EGwItem *item, GHashTable *categor
 	char* value;
 	int element_type;
 	int i;
+	gboolean is_contact_list;
+	
+	is_contact_list = e_gw_item_get_item_type (item) == E_GW_ITEM_TYPE_GROUP ? TRUE: FALSE;
+	e_contact_set (contact, E_CONTACT_IS_LIST, GINT_TO_POINTER (is_contact_list));
+	if (is_contact_list)
+		e_contact_set (contact, E_CONTACT_LIST_SHOW_ADDRESSES, GINT_TO_POINTER (TRUE));
 
-	e_contact_set (contact, E_CONTACT_IS_LIST, e_gw_item_get_item_type (item) == E_GW_ITEM_TYPE_GROUP ? TRUE: FALSE);
 	for ( i = 0; i < num_mappings; i++) {
 		element_type = mappings[i].element_type;
 		if(element_type == ELEMENT_TYPE_SIMPLE)
@@ -1052,6 +1083,7 @@ fill_contact_from_gw_item (EContact *contact, EGwItem *item, GHashTable *categor
 static void
 e_book_backend_groupwise_create_contact (EBookBackend *backend,
 					 EDataBook *book,
+					 guint32 opid,
 					 const char *vcard )
 {
 	EContact *contact;
@@ -1065,8 +1097,7 @@ e_book_backend_groupwise_create_contact (EBookBackend *backend,
 
 	egwb = E_BOOK_BACKEND_GROUPWISE (backend);
 	if (egwb->priv->cnc == NULL) {
-		e_data_book_respond_create(book, GNOME_Evolution_Addressbook_AuthenticationRequired, NULL);
-		return;
+		e_data_book_respond_create(book, opid, GNOME_Evolution_Addressbook_AuthenticationRequired, NULL);
 	}
        
 	contact = e_contact_new_from_vcard(vcard);
@@ -1103,11 +1134,11 @@ e_book_backend_groupwise_create_contact (EBookBackend *backend,
 		e_contact_set (contact, E_CONTACT_UID, id);
 		g_free (id);
 		e_book_backend_summary_add_contact (egwb->priv->summary, contact);
-		e_data_book_respond_create(book,  GNOME_Evolution_Addressbook_Success, contact);
+		e_data_book_respond_create(book, opid, GNOME_Evolution_Addressbook_Success, contact);
 		
 	}
 	else {
-		e_data_book_respond_create(book, GNOME_Evolution_Addressbook_OtherError, NULL);
+		e_data_book_respond_create(book, opid, GNOME_Evolution_Addressbook_OtherError, NULL);
 	}
 	g_object_unref (item);
   
@@ -1116,7 +1147,8 @@ e_book_backend_groupwise_create_contact (EBookBackend *backend,
 static void
 e_book_backend_groupwise_remove_contacts (EBookBackend *backend,
 					  EDataBook    *book,
-					  GList *id_list    )
+					  guint32 opid,
+					  GList *id_list)
 {
   
 	char *id;
@@ -1125,7 +1157,7 @@ e_book_backend_groupwise_remove_contacts (EBookBackend *backend,
 
 	ebgw = E_BOOK_BACKEND_GROUPWISE (backend);
 	if (ebgw->priv->cnc == NULL) {
-		e_data_book_respond_remove_contacts (book, GNOME_Evolution_Addressbook_AuthenticationRequired, NULL);
+		e_data_book_respond_remove_contacts (book, opid, GNOME_Evolution_Addressbook_AuthenticationRequired, NULL);
 		return;
 	}
 	for ( ; id_list != NULL; id_list = g_list_next (id_list)) {
@@ -1134,7 +1166,7 @@ e_book_backend_groupwise_remove_contacts (EBookBackend *backend,
 		deleted_ids =  g_list_append (deleted_ids, id);
 		e_book_backend_summary_remove_contact (ebgw->priv->summary, id);
 	}
-	e_data_book_respond_remove_contacts (book,
+	e_data_book_respond_remove_contacts (book, opid,
 					     GNOME_Evolution_Addressbook_Success,  deleted_ids);
 }
 
@@ -1185,7 +1217,8 @@ set_changes_in_gw_item (EGwItem *new_item, EGwItem *old_item)
 static void
 e_book_backend_groupwise_modify_contact (EBookBackend *backend,
 					 EDataBook    *book,
-					 const char *vcard    )
+					 guint32       opid,
+					 const char   *vcard)
 {	
 	EContact *contact;
 	EBookBackendGroupwise *egwb;
@@ -1200,7 +1233,7 @@ e_book_backend_groupwise_modify_contact (EBookBackend *backend,
 	
 	egwb = E_BOOK_BACKEND_GROUPWISE (backend);
 	if (egwb->priv->cnc == NULL) {
-		e_data_book_respond_modify (book, GNOME_Evolution_Addressbook_AuthenticationRequired, NULL);
+		e_data_book_respond_modify (book, opid, GNOME_Evolution_Addressbook_AuthenticationRequired, NULL);
 		return;
 	}
 	contact = e_contact_new_from_vcard(vcard);
@@ -1231,12 +1264,12 @@ e_book_backend_groupwise_modify_contact (EBookBackend *backend,
 	status = e_gw_connection_get_item (egwb->priv->cnc, egwb->priv->container_id, id,  &old_item);
 
 	if (old_item == NULL) {
-		e_data_book_respond_modify (book, GNOME_Evolution_Addressbook_ContactNotFound, NULL);
+		e_data_book_respond_modify (book, opid, GNOME_Evolution_Addressbook_ContactNotFound, NULL);
 		return;
 	}
 
 	if (status != E_GW_CONNECTION_STATUS_OK) {
-		e_data_book_respond_modify (book, GNOME_Evolution_Addressbook_OtherError, NULL);
+		e_data_book_respond_modify (book, opid, GNOME_Evolution_Addressbook_OtherError, NULL);
 		return;
 	}
 
@@ -1255,12 +1288,12 @@ e_book_backend_groupwise_modify_contact (EBookBackend *backend,
 	e_gw_item_set_item_type (new_item, e_gw_item_get_item_type (old_item));
 	status = e_gw_connection_modify_item (egwb->priv->cnc, id, new_item);
 	if (status == E_GW_CONNECTION_STATUS_OK) {
-		e_data_book_respond_modify (book, GNOME_Evolution_Addressbook_Success, contact);
+		e_data_book_respond_modify (book, opid, GNOME_Evolution_Addressbook_Success, contact);
 		e_book_backend_summary_remove_contact (egwb->priv->summary, id);
 		e_book_backend_summary_add_contact (egwb->priv->summary, contact);
 	}
 	else 
-		e_data_book_respond_modify (book, GNOME_Evolution_Addressbook_OtherError, NULL);
+		e_data_book_respond_modify (book, opid, GNOME_Evolution_Addressbook_OtherError, NULL);
 	g_object_unref (new_item);
 	g_object_ref (old_item);
 	g_object_unref (contact);
@@ -1272,9 +1305,10 @@ e_book_backend_groupwise_modify_contact (EBookBackend *backend,
 static void
 e_book_backend_groupwise_get_contact (EBookBackend *backend,
 				      EDataBook    *book,
-				      const char *id   )
+				      guint32       opid,
+				      const char   *id)
 {
-	EBookBackendGroupwise *gwb ;
+	EBookBackendGroupwise *gwb;
 	int status ;
 	EGwItem *item;
 	EContact *contact;
@@ -1282,7 +1316,7 @@ e_book_backend_groupwise_get_contact (EBookBackend *backend,
 
 	gwb =  E_BOOK_BACKEND_GROUPWISE (backend);
 	if (gwb->priv->cnc == NULL) {
-		e_data_book_respond_get_contact (book, GNOME_Evolution_Addressbook_OtherError, NULL);
+		e_data_book_respond_get_contact (book, opid, GNOME_Evolution_Addressbook_OtherError, NULL);
 		return;
 	}
   	status = e_gw_connection_get_item (gwb->priv->cnc, gwb->priv->container_id, id,  &item);
@@ -1291,7 +1325,7 @@ e_book_backend_groupwise_get_contact (EBookBackend *backend,
 			contact = e_contact_new ();
 			fill_contact_from_gw_item (contact, item, gwb->priv->categories_by_id);
 			vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
-			e_data_book_respond_get_contact (book, GNOME_Evolution_Addressbook_Success, vcard);
+			e_data_book_respond_get_contact (book, opid, GNOME_Evolution_Addressbook_Success, vcard);
 			g_free (vcard);
 			g_object_unref (contact);
 			g_object_unref (item);
@@ -1299,7 +1333,7 @@ e_book_backend_groupwise_get_contact (EBookBackend *backend,
 		}
     
 	}
-	e_data_book_respond_get_contact (book, GNOME_Evolution_Addressbook_ContactNotFound, "");  
+	e_data_book_respond_get_contact (book, opid, GNOME_Evolution_Addressbook_ContactNotFound, "");  
 	
 }
 
@@ -1653,7 +1687,8 @@ e_book_backend_groupwise_build_gw_filter (EBookBackendGroupwise *ebgw, const cha
 static void
 e_book_backend_groupwise_get_contact_list (EBookBackend *backend,
 					   EDataBook    *book,
-					   const char *query )
+					   guint32       opid,
+					   const char   *query )
 {
   
 	GList *vcard_list;
@@ -1671,7 +1706,7 @@ e_book_backend_groupwise_get_contact_list (EBookBackend *backend,
 	gw_items = NULL;
 
 	if (egwb->priv->cnc == NULL) {
-		e_data_book_respond_get_contact_list (book, GNOME_Evolution_Addressbook_AuthenticationRequired, NULL);
+		e_data_book_respond_get_contact_list (book, opid, GNOME_Evolution_Addressbook_AuthenticationRequired, NULL);
 		return;
 	}
 	
@@ -1681,7 +1716,7 @@ e_book_backend_groupwise_get_contact_list (EBookBackend *backend,
 		match_needed = FALSE; 
 	card_sexp = e_book_backend_sexp_new (query);
 	if (!card_sexp) {
-		e_data_book_respond_get_contact_list (book, GNOME_Evolution_Addressbook_InvalidQuery,
+		e_data_book_respond_get_contact_list (book, opid, GNOME_Evolution_Addressbook_InvalidQuery,
 						      vcard_list);
 	}
 
@@ -1702,7 +1737,7 @@ e_book_backend_groupwise_get_contact_list (EBookBackend *backend,
 	}
 
 	if (status != E_GW_CONNECTION_STATUS_OK) {
-		e_data_book_respond_get_contact_list (book, GNOME_Evolution_Addressbook_OtherError,
+		e_data_book_respond_get_contact_list (book, opid, GNOME_Evolution_Addressbook_OtherError,
 						      NULL);
 		return;
 	}
@@ -1718,7 +1753,7 @@ e_book_backend_groupwise_get_contact_list (EBookBackend *backend,
     	}
 	if (gw_items)
 		g_list_free (gw_items);
-	e_data_book_respond_get_contact_list (book, GNOME_Evolution_Addressbook_Success,
+	e_data_book_respond_get_contact_list (book, opid, GNOME_Evolution_Addressbook_Success,
 					      vcard_list);
 	if (filter)
 		g_object_unref (filter);
@@ -1873,6 +1908,7 @@ e_book_backend_groupwise_stop_book_view (EBookBackend  *backend,
 static void
 e_book_backend_groupwise_get_changes (EBookBackend *backend,
 				      EDataBook    *book,
+				      guint32       opid,
 				      const char *change_id  )
 {
 
@@ -1948,6 +1984,7 @@ update_summary (EBookBackendGroupwise *ebgw)
 static void
 e_book_backend_groupwise_authenticate_user (EBookBackend *backend,
 					    EDataBook    *book,
+					    guint32       opid,
 					    const char *user,
 					    const char *passwd,
 					    const char *auth_method)
@@ -1963,7 +2000,7 @@ e_book_backend_groupwise_authenticate_user (EBookBackend *backend,
   
 	priv->cnc = e_gw_connection_new (priv->uri, user, passwd);
 	if (priv->cnc == NULL) {
-		e_data_book_respond_authenticate_user (book,  GNOME_Evolution_Addressbook_AuthenticationFailed);
+		e_data_book_respond_authenticate_user (book, opid, GNOME_Evolution_Addressbook_AuthenticationFailed);
 		return;
 	}
 	id = NULL;
@@ -1974,7 +2011,7 @@ e_book_backend_groupwise_authenticate_user (EBookBackend *backend,
 			status = e_gw_connection_create_book (priv->cnc, priv->book_name,  &id);
 			is_writable = TRUE;
 			if (status != E_GW_CONNECTION_STATUS_OK ) {
-				e_data_book_respond_authenticate_user (book,  GNOME_Evolution_Addressbook_OtherError);
+				e_data_book_respond_authenticate_user (book, opid, GNOME_Evolution_Addressbook_OtherError);
 				return;
 			}
      
@@ -1988,12 +2025,12 @@ e_book_backend_groupwise_authenticate_user (EBookBackend *backend,
 		e_data_book_report_writable (book, is_writable);
 		priv->is_writable = is_writable;
 		e_gw_connection_get_categories (priv->cnc, priv->categories_by_id, priv->categories_by_name);
-		e_data_book_respond_authenticate_user (book,  GNOME_Evolution_Addressbook_Success); 
+		e_data_book_respond_authenticate_user (book, opid, GNOME_Evolution_Addressbook_Success); 
 	
    
 	} else {
 		e_book_backend_set_is_loaded (backend, FALSE);
-		e_data_book_respond_authenticate_user (book, GNOME_Evolution_Addressbook_NoSuchBook);
+		e_data_book_respond_authenticate_user (book, opid, GNOME_Evolution_Addressbook_NoSuchBook);
 	}
              
 	e_util_mkdir_hier (g_path_get_dirname (priv->summary_file_name), 0700);
@@ -2005,7 +2042,8 @@ e_book_backend_groupwise_authenticate_user (EBookBackend *backend,
 
 static void
 e_book_backend_groupwise_get_supported_fields (EBookBackend *backend,
-					       EDataBook    *book )
+					       EDataBook    *book,
+					       guint32       opid)
 {
 	GList *fields = NULL;
 	int i;
@@ -2020,12 +2058,17 @@ e_book_backend_groupwise_get_supported_fields (EBookBackend *backend,
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_IM_JABBER)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_IM_GROUPWISE)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_ADDRESS_WORK)));
-	e_data_book_respond_get_supported_fields (book,
+	e_data_book_respond_get_supported_fields (book, opid,
 						  GNOME_Evolution_Addressbook_Success,
 						  fields);
  
 }
-  
+
+static GNOME_Evolution_Addressbook_CallStatus
+e_book_backend_groupwise_cancel_operation (EBookBackend *backend, EDataBook *book)
+{
+	return GNOME_Evolution_Addressbook_CouldNotCancel;
+}
 
 static GNOME_Evolution_Addressbook_CallStatus
 e_book_backend_groupwise_load_source (EBookBackend           *backend,
@@ -2084,21 +2127,22 @@ e_book_backend_groupwise_load_source (EBookBackend           *backend,
 
 static void
 e_book_backend_groupwise_remove (EBookBackend *backend,
-				 EDataBook        *book)
+				 EDataBook        *book,
+				 guint32           opid)
 {
 	EBookBackendGroupwise *ebgw;
 	int status;
   
 	ebgw = E_BOOK_BACKEND_GROUPWISE (backend);
 	if (ebgw->priv->cnc == NULL) {
-		e_data_book_respond_remove (book,  GNOME_Evolution_Addressbook_OtherError);
+		e_data_book_respond_remove (book,  opid, GNOME_Evolution_Addressbook_OtherError);
 		return;
 	}	
 	status = e_gw_connection_remove_item (ebgw->priv->cnc, NULL, ebgw->priv->container_id);
 	if (status == E_GW_CONNECTION_STATUS_OK) 
-		e_data_book_respond_remove (book,  GNOME_Evolution_Addressbook_Success);
+		e_data_book_respond_remove (book,  opid, GNOME_Evolution_Addressbook_Success);
 	else
-		e_data_book_respond_remove (book,  GNOME_Evolution_Addressbook_OtherError);
+		e_data_book_respond_remove (book,  opid, GNOME_Evolution_Addressbook_OtherError);
     
 }
 
@@ -2116,7 +2160,7 @@ e_book_backend_groupwise_get_static_capabilities (EBookBackend *backend)
 		return g_strdup("net,bulk-removes");
 }
 static void 
-e_book_backend_groupwise_get_supported_auth_methods (EBookBackend *backend, EDataBook *book)
+e_book_backend_groupwise_get_supported_auth_methods (EBookBackend *backend, EDataBook *book, guint32 opid)
 {
 	/*FIXME  provide implementation*/  
 }
@@ -2210,6 +2254,7 @@ e_book_backend_groupwise_class_init (EBookBackendGroupwiseClass *klass)
 	parent_class->authenticate_user       = e_book_backend_groupwise_authenticate_user;
 	parent_class->get_supported_fields    = e_book_backend_groupwise_get_supported_fields;
 	parent_class->get_supported_auth_methods = e_book_backend_groupwise_get_supported_auth_methods;
+	parent_class->cancel_operation        = e_book_backend_groupwise_cancel_operation;
 	parent_class->remove                  = e_book_backend_groupwise_remove;
 	object_class->dispose                 = e_book_backend_groupwise_dispose;
 }
