@@ -30,8 +30,6 @@
 struct ECalListenerPrivate {
 	/* Notification functions and their closure data */
 	ECalListenerCalSetModeFn cal_set_mode_fn;
-	ECalListenerErrorOccurredFn error_occurred_fn;
-	ECalListenerCategoriesChangedFn categories_changed_fn;
 	gpointer fn_data;
 
 	/* Whether notification is desired */
@@ -62,6 +60,8 @@ enum {
 	GET_CHANGES,
 	GET_FREE_BUSY,
 	QUERY,
+	CATEGORIES_CHANGED,
+	BACKEND_ERROR,
 	LAST_SIGNAL
 };
 
@@ -83,8 +83,8 @@ convert_status (const GNOME_Evolution_Calendar_CallStatus status)
 		return E_CALENDAR_STATUS_OBJECT_NOT_FOUND;
 	case GNOME_Evolution_Calendar_InvalidObject:
 		return E_CALENDAR_STATUS_INVALID_OBJECT;
-	case GNOME_Evolution_Calendar_CardIdAlreadyExists:
-		return E_CALENDAR_STATUS_CARD_ID_ALREADY_EXISTS;
+	case GNOME_Evolution_Calendar_ObjectIdAlreadyExists:
+		return E_CALENDAR_STATUS_OBJECT_ID_ALREADY_EXISTS;
 	case GNOME_Evolution_Calendar_AuthenticationFailed:
 		return E_CALENDAR_STATUS_AUTHENTICATION_FAILED;
 	case GNOME_Evolution_Calendar_AuthenticationRequired:
@@ -182,7 +182,7 @@ impl_notifyStaticCapabilities (PortableServer_Servant servant,
 	if (!priv->notify)
 		return;
 
-	g_signal_emit (G_OBJECT (listener), signals[STATIC_CAPABILITIES], 0, convert_status (status));
+	g_signal_emit (G_OBJECT (listener), signals[STATIC_CAPABILITIES], 0, convert_status (status), capabilities);
 }
 
 /* ::notifyCalOpened method */
@@ -308,19 +308,29 @@ impl_notifyObjectsReceived (PortableServer_Servant servant,
 
 static void
 impl_notifyObjectsSent (PortableServer_Servant servant,
-			GNOME_Evolution_Calendar_CallStatus status,
+			const GNOME_Evolution_Calendar_CallStatus status,
+			const GNOME_Evolution_Calendar_UserList *user_list,
+			const CORBA_char *calobj,
 			CORBA_Environment *ev)
 {
 	ECalListener *listener;
 	ECalListenerPrivate *priv;
-	
+	GList *users = NULL;
+	int i;
+
 	listener = E_CAL_LISTENER (bonobo_object_from_servant (servant));
 	priv = listener->priv;
 
 	if (!priv->notify)
 		return;
 
-	g_signal_emit (G_OBJECT (listener), signals[SEND_OBJECTS], 0, convert_status (status));
+	for (i = 0; i < user_list->_length; i++)
+		users = g_list_append (users, g_strdup (user_list->_buffer[i]));
+
+	g_signal_emit (G_OBJECT (listener), signals[SEND_OBJECTS], 0, convert_status (status), users, calobj);
+
+	g_list_foreach (users, (GFunc) g_free, NULL);
+	g_list_free (users);
 }
 
 static void 
@@ -610,8 +620,6 @@ impl_notifyCalSetMode (PortableServer_Servant servant,
 	if (!priv->notify)
 		return;
 
-	g_message ("notify_set_mode");
-
 	g_assert (priv->cal_set_mode_fn != NULL);
 	(* priv->cal_set_mode_fn) (listener, status, mode, priv->fn_data);
 }
@@ -632,10 +640,7 @@ impl_notifyErrorOccurred (PortableServer_Servant servant,
 	if (!priv->notify)
 		return;
 
-	g_message ("notify_error");
-
-	g_assert (priv->error_occurred_fn != NULL);
-	(* priv->error_occurred_fn) (listener, message, priv->fn_data);
+	g_signal_emit (G_OBJECT (listener), signals[BACKEND_ERROR], 0, message);
 }
 
 /* ::notifyCategoriesChanged method */
@@ -653,10 +658,7 @@ impl_notifyCategoriesChanged (PortableServer_Servant servant,
 	if (!priv->notify)
 		return;
 
-	g_message ("notify_categories");
-
-	g_assert (priv->categories_changed_fn != NULL);
-	(* priv->categories_changed_fn) (listener, categories, priv->fn_data);
+	g_signal_emit (G_OBJECT (listener), signals[CATEGORIES_CHANGED], 0, categories);
 }
 
 
@@ -670,9 +672,6 @@ e_cal_listener_init (ECalListener *listener, ECalListenerClass *klass)
 	priv = g_new0 (ECalListenerPrivate, 1);
 	listener->priv = priv;
 
-	priv->error_occurred_fn = NULL;
-	priv->categories_changed_fn = NULL;
-
 	priv->notify = TRUE;
 }
 
@@ -689,8 +688,6 @@ e_cal_listener_finalize (GObject *object)
 	listener = E_CAL_LISTENER (object);
 	priv = listener->priv;
 
-	priv->error_occurred_fn = NULL;
-	priv->categories_changed_fn = NULL;
 	priv->fn_data = NULL;
 
 	priv->notify = FALSE;
@@ -842,8 +839,8 @@ e_cal_listener_class_init (ECalListenerClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (ECalListenerClass, send_objects),
 			      NULL, NULL,
-			      e_cal_marshal_VOID__INT,
-			      G_TYPE_NONE, 1, G_TYPE_INT);
+			      e_cal_marshal_VOID__INT_POINTER_STRING,
+			      G_TYPE_NONE, 3, G_TYPE_INT, G_TYPE_POINTER, G_TYPE_STRING);
 	signals[DEFAULT_OBJECT] =
 		g_signal_new ("default_object",
 			      G_TYPE_FROM_CLASS (klass),
@@ -899,7 +896,7 @@ e_cal_listener_class_init (ECalListenerClass *klass)
 			      G_STRUCT_OFFSET (ECalListenerClass, get_changes),
 			      NULL, NULL,
 			      e_cal_marshal_VOID__INT_POINTER,
-			      G_TYPE_NONE, 1, G_TYPE_INT, G_TYPE_POINTER);
+			      G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_POINTER);
 	signals[GET_FREE_BUSY] =
 		g_signal_new ("get_free_busy",
 			      G_TYPE_FROM_CLASS (klass),
@@ -907,7 +904,7 @@ e_cal_listener_class_init (ECalListenerClass *klass)
 			      G_STRUCT_OFFSET (ECalListenerClass, get_free_busy),
 			      NULL, NULL,
 			      e_cal_marshal_VOID__INT_POINTER,
-			      G_TYPE_NONE, 1, G_TYPE_INT, G_TYPE_POINTER);
+			      G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_POINTER);
 	signals[QUERY] =
 		g_signal_new ("query",
 			      G_TYPE_FROM_CLASS (klass),
@@ -916,6 +913,22 @@ e_cal_listener_class_init (ECalListenerClass *klass)
 			      NULL, NULL,
 			      e_cal_marshal_VOID__INT_POINTER,
 			      G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_POINTER);
+	signals[CATEGORIES_CHANGED] =
+		g_signal_new ("categories_changed",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ECalListenerClass, categories_changed),
+			      NULL, NULL,
+			      e_cal_marshal_VOID__POINTER,
+			      G_TYPE_NONE, 1, G_TYPE_POINTER);
+	signals[BACKEND_ERROR] =
+		g_signal_new ("backend_error",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ECalListenerClass, backend_error),
+			      NULL, NULL,
+			      e_cal_marshal_VOID__STRING,
+			      G_TYPE_NONE, 1, G_TYPE_STRING);
 }
 
 BONOBO_TYPE_FUNC_FULL (ECalListener,
@@ -925,24 +938,18 @@ BONOBO_TYPE_FUNC_FULL (ECalListener,
 
 ECalListener *
 e_cal_listener_construct (ECalListener *listener,
-			ECalListenerCalSetModeFn cal_set_mode_fn,
-			ECalListenerErrorOccurredFn error_occurred_fn,
-			ECalListenerCategoriesChangedFn categories_changed_fn,
-			gpointer fn_data)
+			  ECalListenerCalSetModeFn cal_set_mode_fn,
+			  gpointer fn_data)
 {
 	ECalListenerPrivate *priv;
 
 	g_return_val_if_fail (listener != NULL, NULL);
 	g_return_val_if_fail (E_IS_CAL_LISTENER (listener), NULL);
  	g_return_val_if_fail (cal_set_mode_fn != NULL, NULL);
-	g_return_val_if_fail (error_occurred_fn != NULL, NULL);
-	g_return_val_if_fail (categories_changed_fn != NULL, NULL);
 
 	priv = listener->priv;
 
 	priv->cal_set_mode_fn = cal_set_mode_fn;
-	priv->error_occurred_fn = error_occurred_fn;
-	priv->categories_changed_fn = categories_changed_fn;
 	priv->fn_data = fn_data;
 
 	return listener;
@@ -952,9 +959,6 @@ e_cal_listener_construct (ECalListener *listener,
  * e_cal_listener_new:
  * @cal_set_mode_fn: Function callback for notification that a
  * calendar changed modes
- * @error_occurred_fn:  Function that will be called to notify errors.
- * @categories_changed_fn: Function that will be called to notify that the list
- * of categories that are present in the calendar's objects has changed.
  * @fn_data: losure data pointer that will be passed to the notification
  * functions.
  * 
@@ -964,14 +968,11 @@ e_cal_listener_construct (ECalListener *listener,
  **/
 ECalListener *
 e_cal_listener_new (ECalListenerCalSetModeFn cal_set_mode_fn,
-		  ECalListenerErrorOccurredFn error_occurred_fn,
-		  ECalListenerCategoriesChangedFn categories_changed_fn,
-		  gpointer fn_data)
+		    gpointer fn_data)
 {
 	ECalListener *listener;
 
-	g_return_val_if_fail (error_occurred_fn != NULL, NULL);
-	g_return_val_if_fail (categories_changed_fn != NULL, NULL);
+	g_return_val_if_fail (cal_set_mode_fn != NULL, NULL);
 
 	listener = g_object_new (E_TYPE_CAL_LISTENER, 
 				 "poa", bonobo_poa_get_threaded (ORBIT_THREAD_HINT_PER_REQUEST, NULL),
@@ -979,8 +980,6 @@ e_cal_listener_new (ECalListenerCalSetModeFn cal_set_mode_fn,
 
 	return e_cal_listener_construct (listener,
 					 cal_set_mode_fn,
-					 error_occurred_fn,
-					 categories_changed_fn,
 					 fn_data);
 }
 
